<?php
/* $Id$ */
/*
	xgate.inc
	Copyright (C) 2006-2012 Global Marine Networks
	All rights reserved.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:

	1. Redistributions of source code must retain the above copyright notice,
	   this list of conditions and the following disclaimer.

	2. Redistributions in binary form must reproduce the above copyright
	   notice, this list of conditions and the following disclaimer in the
	   documentation and/or other materials provided with the distribution.

	THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
	INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
	AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
	AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
	OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
	SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
	INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
	CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
	POSSIBILITY OF SUCH DAMAGE.
*/

require_once('globals.inc');
require_once('config.inc');
require_once('util.inc');
require_once('pfsense-utils.inc');
require_once('pkg-utils.inc');
require_once('service-utils.inc');
require_once('vhosts.inc');

if(!function_exists("filter_configure")) 
	require_once("filter.inc");

define('XGATE', '/usr/local/gmnbin/xgate');
define('RC', '/usr/local/etc/rc.d/');
define('XGATERC', RC . 'xgate.sh');
define('XGATEMON', RC . 'email_monitor.sh');
define('XGATE_CONFBASE', '/usr/local/etc/xgate/');
define('XGATE_VAR', '/var/xgate/');
define('XGATE_SPOOL', '/var/xgate/spool/');
define('XGATE_INI', XGATE_VAR . 'XGate.Ini');
define('XGATE_NET_INI', XGATE_VAR . 'Net.Ini');
define('XGATE_ALIAS_INI', XGATE_VAR . 'XGate.Alias');
define('XGATE_LIST_INI', XGATE_VAR . 'XGate.List');
define('XGATE_XMAILFILT', XGATE_VAR . 'xmailfilt.dat');
define('XGATE_USER', XGATE_VAR . 'XGate.User');
define('XGATE_PASS', XGATE_VAR . 'XGate.Pass');
define('XGATE_SUBACCT', XGATE_VAR . 'xsubacc.dat');
define('XGATE_OUTBOX', XGATE_SPOOL . 'OutBox/');
define('XGATE_TMP', XGATE_SPOOL . 'tmp/');
define('XGATE_BIGMAIL', XGATE_VAR . 'xmailbot.dat');
define('XGATE_PID', XGATE_VAR . 'pid');

define('T_DAH', '/usr/local/vhosts/webmail/');
define('T_DAH_DB', XGATE_SPOOL . 'database/');
define('T_DAH_BASEINC', T_DAH . 'inc/');
define('T_DAH_DB_DIST', T_DAH_BASEINC . 'database');
define('T_DAH_BASE',  T_DAH_BASEINC . 'config/');
define('T_DAH_MAILCONF', T_DAH_BASE . 'config.mail.php');
define('T_DAH_MAILCONF_TMPL', T_DAH_MAILCONF . '.template');
define('T_DAH_CONF', T_DAH_BASE . 'config.php');
define('T_DAH_CONF_TMPL', T_DAH_CONF . '.template');

//
// Connection settings
//

function xgate_before_form_connection($pkg) 
{
	$conf = xg_parse_ini_file(XGATE_NET_INI);
	$servers = array_keys($conf['HOSTS']);
	$ports = array_keys($conf['PORTS']);
	
	$isps = $conf['ISPS'];
	foreach( $isps as $key => $val ) {
	    $isp = $conf[trim($val)];
	    $proto = trim($isp['ipproto']);
	    if ( $proto == 'wxa' || $proto == 'at' || $proto == 'network' ) {
	        $networks[] = trim($val);
		$types[] = trim($key);
	    }
	}

	xgate_build_options($pkg, 'gateway_xport', $ports, $ports);
	xgate_build_options($pkg, 'gateway_gateway', $servers, $servers);
	xgate_build_options($pkg, 'gateway_gateway2', $servers, $servers);	
	xgate_build_options($pkg, 'connection_connection', $networks, $types);
}

function xgate_validate_connection($post, $input_errors) 
{
	global $config;
	$settings = $config['installedpackages']['xgate']['config'][0];

	$failover = $post['gateway_failover'];
	$gateway  = $post['gateway_gateway'];
	$gateway2 = $post['gateway_gateway2'];
	$bind     = $post['gateway_bind_static'];
	$if       = $post['gateway_primary_interface'];
	$if2	  = $post['gateway_secondary_interface'];
	$override = $post['gateway_b_timeout_override'];
	$gwconto  = intval(trim($post['gateway_connection_timeout']));
	$gwxferto = intval(trim($post['gateway_xfer_timeout']));
	$persist  = $post['connection_persist'];
	$numatt   = intval(trim($post['connection_attempts']));
	$sec      = intval(trim($post['connection_secs']));
	$ipdialoverride = trim($post['connection_ipdialoverride']);

	$conf = xg_parse_ini_file(XGATE_NET_INI);
	$gatewayip  = $conf['HOSTS'][$gateway];
	$gateway2ip = $conf['HOSTS'][$gateway2];

	if ( !empty($ipdialoverride) && !is_ipaddr($ipdialoverride) ) {
	    $input_errors[] = "IP dial address must be a valid IP address ($ipdialoverride).";
	}

	if ( $bind == 'on' && $if == $if2 ) {
	    $input_errors[] = "Primary mail interface ($if) must be different than the secondary interface ($if2)";
	} else {
	    $primaryip = get_interface_gateway($if);
            $secondaryip = get_interface_gateway($if2);

	    if ( !is_ipaddr($primaryip) || !is_ipaddr($secondaryip ) ) {
	        $input_errors[] = "Both WAN ports ($if:{$primaryip} and $if2:{$secondaryip}) must be configured and have valid IP addresses";
	    }
	}

	if ( $failover == 'on' && $gateway == $gateway2 ) {
	    $input_errors[] = "You must select two different gateways when enabling failover";
	}

	if ( $failover == 'on' && $gatewayip == $gateway2ip ) {
	    $input_errors[] = "The IP addresses of the selected gateways are the same (gateway = $gatewayip, gateway2 = $gateway2ip). Please try a different secondary gateway.";
	}

	if ( $override == 'on' && !is_uint($gwconto) ) {
	    $input_errors[] = "Connection timeout ($gwconto)  must be a positive integer";
	}

	if ( $override == 'on' && !is_uint($gwxferto) ) {
	    $input_errors[] = "Send/Receive timeout ($gwxferto) must be a positive integer";
	}

	if ( $persist == 'on' && !is_uint($numatt) ) {
	    $input_errors[] = "Number of attempts must be a positive integer";
	}
	
	if ( $persist == 'on' && !is_uint($sec) ) {
	    $input_errors[] = "Seconds to wait ($sec) must be a positive integer";
	}
}

function xgate_resync_connection() 
{
	global $g, $config, $valid_acls;

	$settings = $config['installedpackages']['xgateconnection']['config'][0];

	$conf = array('Gateway' => array(), 'Connection' => array());
	
	$xport          = $settings['gateway_xport'];
	$gateway        = $settings['gateway_gateway'];
	$failover       = $settings['gateway_failover'];
	$gateway2       = $settings['gateway_gateway2'];
	$bind	        = $settings['gateway_bind_static'];
	$if	        = $settings['gateway_primary_interface'];
	$if2	        = $settings['gateway_secondary_interface'];
	$connection     = $settings['connection_connection'];
	$dialoveride    = $settings['connection_dialoveride'];
	$ipdialoverride = $settings['connection_ipdialoverride'];
	$leaveopen      = $settings['connection_leaveopen'];
	$useifopen      = $settings['connection_useifopen'];
	$override       = $settings['gateway_b_timeout_override'];
	$gwconto        = $settings['gateway_connection_timeout'];
	$gwxferto       = $settings['gateway_xfer_timeout'];
	$persist        = $settings['connection_persist'];
	$numatt         = $settings['connection_attempts'];
	$sec            = $settings['connection_secs'];

        $primaryip = get_interface_gateway($if);
        $secondaryip = get_interface_gateway($if2);

	$ini = xg_parse_ini_file(XGATE_NET_INI);

        $gwip  = trim($ini['HOSTS'][$gateway]);
        $gw2ip = trim($ini['HOSTS'][$gateway2]);

	if ( $bind == 'on' ) { 
             mwexec("/sbin/route delete -host {$gwip}  2>&1", true);
             mwexec("/sbin/route add -host {$gwip} {$primaryip} 2>&1", true);
             mwexec("/sbin/route delete -host {$gwip2} 2>&1", true);
             mwexec("/sbin/route add -host {$gw2ip} {$secondaryip} 2>&1", true);
       	} else {
      	     mwexec("/sbin/route delete -host {$gwip}  2>&1", true);
             mwexec("/sbin/route delete -host {$gw2ip} 2>&1", true);
	}
	$conf['Gateway']['xport'] = $xport;
	$conf['Gateway']['gateway'] = $gateway;
	$conf['Gateway']['failover'] = $failover == 'on' ? 1 : 0;
	if ( $failover == 'on' ) {
	    $conf['Gateway']['gateway2'] = $gateway2;
	}

	$conf['Connection']['connection'] = "Network Connection";
	$conf['Connection']['type'] = $connection;
	$conf['Connection']['dialoveride'] = empty($dialoveride) ? 0 : 1;
	if ( !empty($dialoveride) ) {
	    $conf['Connection']['phonenum'] = $dialoveride;
	}

	$conf['Connection']['ipdialoverride'] = empty($ipdialoverride) ? 0 : 1;
	if ( !empty($ipdialoverride) ) {
	    $conf['Connection']['ipdialaddr'] = $ipdialoverride;
	} else {
	    $network = $ini['ISPS'][$connection];
	    $isp = $ini[$network];
	    switch ( $isp['ipproto'] ) {
	        case "wxa":
		    $conf['Connection']['ipdialoverride'] = 1;
		    $conf['Connection']['ipdialaddr'] = "127.0.0.1";
		    break;
		case "at":
		    exec("/usr/bin/netstat -r -n | grep default | awk '{print $2}'", $defroute);
		    $conf['Connection']['ipdialoverride'] = 1;
		    $conf['Connection']['ipdialaddr'] = trim($defroute[0]);
	            break;
		default:
		    break;
	    }
	}
	$conf['Connection']['leaveopen'] = $leaveopen == 'on' ? 1 : 0;
	$conf['Connection']['useifopen'] = $useifopen == 'on' ? 1 : 0;
	$conf['Gateway']['bTimeOutOverride'] = $override == 'on' ? 1 : 0;
	if ( $override == 'on' ) {
	    $conf['Gateway']['connection_timeout'] = $gwconto;
   	    $conf['Gateway']['xfer_timeout'] = $gwxferto;
	}
	$conf['Connection']['persist'] = $persist == 'on' ? 1 : 0;
	if ( $persist == 'on' ) {
	    $conf['Connection']['attempts'] = $numatt;
	    $conf['Connection']['secs'] = $sec;
	}

	return $conf;
}


//
// sub accounts
//
function xgate_before_form_sub_accounts($pkg) 
{
	$users = get_primary_users();
	xgate_build_options($pkg, 'xgatesubaccountsprimary', $users, $users);
}

function xgate_validate_sub_accounts($post, $input_errors) 
{
	global $config;
	$settings = $config['installedpackages']['xgatesubaccounts']['config'];

	$username = trim($post['username']);
	$password = trim($post['password']);
	$xgatesubaccountsprimary = trim($post['xgatesubaccountsprimary']);

	$indx = 0;
	foreach ($settings as $acct) {
	    if ( $acct['username'] == $username && $acct['xgatesubaccountsprimary'] == $xgatesubaccountsprimary ) {
	        unset($settings[$indx]);
		break;
	    }
	    $indx++;
	}

	if ( empty($xgatesubaccountsprimary) ) {
	    $input_errors[] = "No primary account selected";
  	}

	if ( !is_validuid($username) ) {
	    $input_errors[] = "Invalid username. Valid userids can contain only the following characters [a-zA-Z0-9._-]";
 	}
	
	if ( !is_validpw($password) ) {
	    $input_errors[] = "Invalid password. Passwords must be 6 characters or longer";
	}	
}

function xgate_delete_sub_accounts()
{
	xgate_resync_sub_accounts();
	xgate_purge_spool();
}

function xgate_resync_sub_accounts()
{
	global $g, $config;
	$settings = $config['installedpackages']['xgatesubaccounts']['config'];

	$subs = array();

	$primaryusers = get_primary_users();
        if ( count($settings) ) {
	    foreach ($settings as $acct) {
	        $primary = $acct['xgatesubaccountsprimary'];
	        if ( in_array($primary, $primaryusers) ) {
	    	    $subs[$primary][$acct['username']]=$acct['password'];
	        }
	    }
	}
	
	$keys = array_keys($subs);
	$indx = 0;
	foreach($keys as $key) {
	    $subs[$key]['Count'] = count($subs[$key]);
	    $subs['Accounts']["User{$indx}"] = $key;
	    $indx++;
	}
	$subs['Accounts']['Count'] = count($subs['Accounts']);	
	write_php_ini($subs, XGATE_SUBACCT);
}

function get_sub_users($puser) 
{
    global $g, $config;
    $settings = $config['installedpackages']['xgatesubaccounts']['config'];

    $ret = array();
    foreach ($settings as $acct) {
        $primary = $acct['xgatesubaccountsprimary'];
	if ( $primary == $puser ) {
	    $ret[] = $acct['username'];
	}
    }
    return $ret;
}

function xgate_del_sub($primaryact)
{
    global $g, $config;
    $settings = &$config['installedpackages']['xgatesubaccounts']['config'];
    
    $pusers = ($primaryact == "All")  ? get_primary_users() : array($primaryact);
    
    foreach ($settings as $key => $acct) {
        if ( in_array($acct['xgatesubaccountsprimary'], $pusers) ) {
	    unset($settings[$key]);
	}
    }

    write_config();
    xgate_resync_sub_accounts();
    xgate_purge_spool();
}

function xgate_broadcast_msg($primaryact, $subject, $msg)
{
    global $g, $config;

    $xgate = $config['installedpackages']['xgate']['config'][0];
    $domain = $xgate['sat_domain'];
    $from = 'system@' . $domain;
    $date = date('r');

    $mail = <<<EOD
To: undisclosed-recipient
From: $from
Date: $date
Subject: $subject

$msg

EOD;

    $tname = tempnam("/tmp", "MAIL");
    $h = fopen($tname, "w");
    fwrite($h, $mail);
    fclose($h);
    $fname = basename($tname) . ".Mail";

    $pusers = ($primaryact == "All")  ? get_primary_users() : array($primaryact);

    foreach ( $pusers as $puser ) {
	$dst = XGATE_SPOOL . $puser;
        if ( !file_exists($dst) ) { 
	    mkdir ($dst);
	}
	$dstmail = $dst . '/' . $fname;
        copy( $tname, $dstmail);
	$subusers = get_sub_users($puser);

	foreach ( $subusers as $subuser) {
	    $subdst = $dst . '/' . $subuser;	
            if ( !file_exists($subdst) ) {
	        mkdir($subdst);
	    }
	    $dstmail = $subdst . '/' . $fname;
            copy( $tname, $dstmail);
	}
    }
    unlink($tname);
}

function xgate_import_sub($primaryact, $file)
{
    global $config;

    if ( !file_exists($file) ) {
        return "Import failed. $file does not exist";
    }

    $pusers = get_primary_users();

    if ( !in_array($primaryact, $pusers) ) {
        return "Primary account $primaryact does not exist";
    }

    // normally we would use file here but it does not work with 
    // mac delimited text files
    $fd = fopen($file,"r");
    $s_csv= fread($fd, filesize($file));
    fclose($fd);
    if ( strpos($s_csv, "\r\n") ) {
        $csv = explode("\r\n", $s_csv);
    } else if ( strpos($s_csv,"\r") ) {
        $csv = explode("\r",  $s_csv);
    } else {
        $csv = explode("\n", $s_csv);
    }

    $susers = get_sub_users($primaryact);

    $settings =  &$config['installedpackages']['xgatesubaccounts']['config'];
    foreach ($csv as $line) {
	$line = trim($line);
        $line = str_replace("\"", "", $line );
        $line = str_replace("\'", "", $line );
	$rec = explode(',', $line);
	
	// check for dups
	if ( !in_array($rec[0], $susers) ) {
	    $conf = array('xgatesubaccountsprimary' => $primaryact, 'username' => $rec[0], 'password' => $rec[1], 'description' => $rec[2]);
	    $settings[] = $conf;
	}
    }

    write_config();
    xgate_resync_sub_accounts();
    xgate_purge_spool();
    return true;
}

//
// primary accounts
//
function xgate_before_form_primary_accounts($pkg) 
{
}

function xgate_validate_primary_accounts($post, $input_errors) 
{
	global $config;
	$settings = $config['installedpackages']['xgateprimaryusers']['config'][0];
	$xgate    = $config['installedpackages']['xgate']['config'][0];

	$username = trim($post['username']);
	$password = trim($post['password']);

	if ( $username == $xgate['gateway_username'] ) {
	    $input_errors[] = "Username same as main identity.";
	}

	$indx = 0;
	foreach ($settings as $acct) {
	    if ( $acct['username'] == $username ) {
		uset($settings[$indx]);
		break;
	    }
	    $indx++;
	}

	if ( !is_validuid($username) ) {
	    $input_errors[] = "Invalid username. Valid userids can contain only the following characters [a-zA-Z0-9._-]";
 	}
	
	if ( !is_validpw($password) ) {
	    $input_errors[] = "Invalid password. Passwords must be 6 characters or longer";
	}	
}

function xgate_delete_primary_accounts()
{
	xgate_resync_primary_accounts();
	xgate_purge_spool();
}

function xgate_resync_primary_accounts()
{
	global $g, $config;
	$settings = $config['installedpackages']['xgateprimaryusers']['config'];
	$xgate    = $config['installedpackages']['xgate']['config'][0];
	$users = array('User' => array() );
	$passwd = array('Passwords' => array() );

	$mainun = $xgate['gateway_username'];
	$mainpw = $xgate['gateway_password'];
	$users['User']['User0'] = $mainun;
	$indx = 0;
	if ( !empty($mainun) && !empty($mainpw) ) {
		$passwd['Passwords'][$mainun] = $mainpw;
		$indx = 1;
	}
        if ( count($settings) ) {
	    foreach ($settings as $acct) {
	        $users['User']["User{$indx}"] = $acct['username'];
	        $passwd['Passwords'][$acct['username']] = $acct['password'];
	        $indx++;
	    }
	}
	$users['User']['Count'] = $indx;
	write_php_ini($users, XGATE_USER);
	write_php_ini($passwd, XGATE_PASS);
}


//
// filter
//
function xgate_before_form_filters($pkg) 
{
	$users = get_primary_users();
	$cutoff = array('10', '25', '35', '50', '75', '100', '150', '200', '250', '300', '400', '500', '1000', '2000', '4000', '8000');
	xgate_build_options($pkg, 'filters_primary_users', $users, $users);
	xgate_build_options($pkg, 'mailfilter_cutoffsize', $cutoff, $cutoff);
	xgate_build_options($pkg, 'mailfilter_outcutoffsize', $cutoff, $cutoff);
	xgate_build_options($pkg, 'filters_purge_cutoff', $cutoff, $cutoff);
}

function xgate_validate_filters($post, $input_errors) 
{
	global $config;
	$settings = $config['installedpackages']['xgate']['config'][0];

	$user       = $post['filters_primary_users'];
	$incutoff   = $post['filters_inbound_cutoff'];
        $insz       = $post['mailfilter_cutoffsize'];
	$outcutoff  = $post['filters_outbound_cutoff'];
	$outsz      = $post['mailfilter_outcutoffsize'];
        $purgerst   = $post['filters_purge_restart'];
	$mvbm       = $post['filters_move_back_to_bigmail'];
	$syncbml    = $post['filters_sync_bigmail'];
	$purge      = $post['filters_purge_inbox'];
	$purgebm    = $post['filters_purge_bigmail'];
	$purgeibwc  = $post['filters_purge_inbox_with_cutoff'];
	$purgectoff = $post['filters_purge_cutoff'];

	$domain	    = $settings['sat_domain'];

	$conf = xg_parse_ini_file(XGATE_INI);

	if ( $incutoff == 'on' )  $conf['MailFilter']['cutoffsize'] = $insz;
	if ( $outcutoff == 'on' ) $conf['MailFilter']['outcutoffsize'] = $outsz;

	if ( $incutoff != 'on') $insz = $conf['MailFilter']['cutoffsize'];
	if ( empty($insz) ) $insz = 50;

	if ( $purgerst  == 'on' ) bm_purge_restart_files();
	if ( $incutoff  == 'on' ) bm_inbound_cutoff($user,$insz);
	if ( $mvbm      == 'on' ) bm_move_back($user, $insz);
	if ( $syncbml   == 'on' ) bm_sync($user);
	if ( $purge     == 'on' ) bm_purge_inbox($user);
	if ( $purgebm   == 'on' ) bm_purge_bm($user);
	if ( $purgeibwc == 'on' ) bm_purge_inbox_with_cutoff($user,$purgectoff);

	write_php_ini($conf, XGATE_INI);

	// reset the form. these filters are shoot and run
	write_config();
}

function xgate_resync_filters()
{

}


//
// mail filtering
//
function xgate_validate_mailfilter($post, $input_errors) 
{
	global $config;
	$settings = $config['installedpackages']['xgate']['config'][0];
	
	$wl = $post['misc_usewhitelistonsend'];
	$list = $post['mailfilter_email_white_list'];

	if ( $wl == 'on' && empty($list) ) {
	    $input_errors[] = "Email restrictions are enabled but no addresses have been specified.";
	}
}

function xgate_resync_mailfilter()
{
	global $g, $config, $valid_acls;
	$settings = $config['installedpackages']['xgate']['config'][0];
	
	$conf = array('Misc' => array());
	
	$bmsend = $settings['misc_allowplusbig'];
	$wl = $settings['misc_usewhitelistonsend'];
	$bmnotify = $settings['misc_bigmailnotifyenum'];
	$conf['Misc']['allowplusbigmailonsend'] = $bmsend == 'on' ? 0 : 1;
	$conf['Misc']['usewhitelistonsend'] = $wl == 'on' ? 1 : 0;
	$conf['Misc']['bigmailnotifyenum'] = $bmnotify;
	
        if ( $wl == "on" ) {
   	    $val = explode(";", $settings['mailfilter_email_white_list']);
	    $fh = fopen(XGATE_XMAILFILT, "w");
	    for( $i = 0, $j = 0; $i < count($val); $i++ ) {
	        $line = trim($val[$i]);
	        if ( !empty($line) ) {
		    $out[$j++] = $line;
	        }
            }
	    fwrite($fh, count($out) . "\n");
	    if ( $out ) {	 
	        foreach($out as $key => $value) {
	            fwrite($fh, "{$value}\n");
	        }
            }
	    fclose($fh);
	}
	return $conf;
}

//
// XGate general settings
function xgate_before_form_general($pkg) 
{

}

function xgate_validate_general($post, $input_errors) 
{
	global $config;
	$settings = $config['installedpackages']['xgate']['config'][0];
	$username  = trim($post['gateway_username']);
	$password  = trim($post['gateway_password']);
	$domain    = trim($post['sat_domain']);
	$relay     = $post['sat_relayflag'];
	$relayaddr = trim($post['sat_mailrelayaddr']);
	$pop3port = intval(trim($post['mail_pop3port']));
	$smtpport = intval(trim($post['mail_smtpport']));
	$interval = intval(trim($post['tracking_automatic_updates']));
	$enabled  = $post['misc_enabled'];

	$webmail_idle = intval(trim($post['webmail_idletimeout']));
	$webmail_sig  = $post['webmail_signature'];
	$webmail_ban  = $post['webmail_banner'];

	if ( $enabled != "on" ) {
		return;
	}

	if ( empty($domain) ) {
	    $input_errors[] = "A domain must be specified";
	}

	if ( !is_uint($webmail_idle) ) {
	    $input_errors[] = "Idle timeout ($webmail_idle) must be a positive integer";
	}

	if ( empty($webmail_sig) ) {
	    $input_errors[] = "An email signature must be specified";
	}

	if ( empty($webmail_ban) ) {
	    $input_errors[] = "A login banner must be specified";
	}

	if ( !is_validuid($username) ) {
	    $input_errors[] = "Invalid main identity. Valid userids can contain only the following characters [a-zA-Z0-9._-]";
	}

	if ( !is_validpw($password) ) {
	    $input_errors[] = "Invalid password. Passwords must be 6 characters or longer";
	}

	if ( $relay == 'on' && !is_ipaddr($relayaddr) ) {
	    $input_errors[] = "Invalid realy network address. Hint: try 0.0.0.0";
	}

	if ( !is_uint($pop3port) ) {
	    $input_errors[] = "POP3 Port ($pop3port)  must be a positive integer";
	}

	if ( !is_uint($smtpport) ) {
	    $input_errors[] = "SMTP Port ($smtpport) must be a positive integer";
	}

	if ( !is_uint($interval) || $interval == 0 ) {
	    $input_errors[] = "Update interval must be a positive integer";
	}

	xgate_validate_mailfilter($post, &$input_errors);

}

function xgate_resync_general() 
{
	global $g, $config, $valid_acls;

	$settings = $config['installedpackages']['xgate']['config'][0];
	
	$conf = array('All' => array(), 'POP3' => array(), 'SMTP' => array(), 'Gateway' => array(), 'Sat' => array(), 'Mail' => array());
	
        $username  = $settings['gateway_username'];
        $password  = $settings['gateway_password'];
        $domain    = $settings['sat_domain'];
        $relay     = $settings['sat_relayflag'];
        $relayaddr = $settings['sat_mailrelayaddr'];
        $pop3port = $settings['mail_pop3port'];
        $smtpport = $settings['mail_smtpport'];
	$poplog   = $settings['mail_poplog'];
	$smtplog  = $settings['mail_smtplog'];
	$interval = $settings['tracking_automatic_updates'];
	$async    = $settings['mail_concurrency'];
	$enabled  = $settings['misc_enabled'];

	$webmail_idle = $settings['webmail_idletimeout'];
	$webmail_sig  = $settings['webmail_signature'];
	$webmail_ban  = $settings['webmail_banner'];

	$logrotate    = $settings['mail_logrotate'];
	$logdepth     = $settings['misc_logdepth'];

	system("cat ".T_DAH_CONF_TMPL." | sed -e 's/IDLETIMEOUT/$webmail_idle/' | sed -e 's/LOGINBANNER/$webmail_ban/' | sed -e 's/POPPORT/$pop3port/' | sed -e 's/SIGNATURE/$webmail_sig/' > ".T_DAH_CONF);
	system("cat ".T_DAH_MAILCONF_TMPL." | sed -e 's/DOMAIN/$domain/' | sed -e 's/POPPORT/$pop3port/' | sed -e 's/SMTPPORT/$smtpport/' > ".T_DAH_MAILCONF);

	xgate_install_cron(true, $logrotate);

	if ( empty($relayaddr) ) {
		$relayaddr = "0.0.0.0";
	}
	$conf['Gateway']['username'] = $username;
	$conf['Gateway']['password'] = $password;
	$conf['Sat']['domain'] = $domain;
	$conf['All']['Domain0'] = $domain;
	$conf['Sat']['relayflag'] = $relay == 'on' ? 1 : 0;
	$conf['Sat']['MailRelayAddr'] = $relayaddr;
	$conf['Mail']['pop3port'] = $pop3port;
	$conf['POP3']['Port'] = $pop3port;
	$conf['Mail']['smtpport'] = $smtpport;
	$conf['SMTP']['Port'] = $smtpport;
	$conf['SMTP']['Relay'] = $conf['Sat']['relayflag'];
	$conf['Mail']['poplog'] = $poplog == 'on' ? 1 : 0;
	$conf['Mail']['smtplog'] = $smtplog == 'on' ? 1 : 0;
	$conf['Tracking']['AutomaticUpdates'] = "1";
	$conf['Tracking']['UpdateInterval'] = $interval;
	$conf['Mail']['concurrency'] = $async == 'on' ? 1 : 0;
	$conf['Misc']['enabled'] = $enabled == 'on' ? 1 : 0;
	$conf['Misc']['logdepth'] = $logdepth;
	return $conf;
}


//
// global functions
//
function MergeArrays($Arr1, $Arr2) 
{
  foreach($Arr2 as $key => $Value) {
    if(array_key_exists($key, $Arr1) && is_array($Value))
      $Arr1[$key] = MergeArrays($Arr1[$key], $Arr2[$key]);
    else
      $Arr1[$key] = $Value;
  }
  return $Arr1;
}

function xgate_resync() {
	global $config;

	$conf = xg_parse_ini_file(XGATE_INI);
	$conf = MergeArrays($conf, xgate_resync_general());
	$conf = MergeArrays($conf, xgate_resync_connection());
	$conf = MergeArrays($conf, xgate_resync_mailfilter());
	xgate_resync_primary_accounts();
	xgate_resync_sub_accounts();

	write_php_ini($conf, XGATE_INI);

	if (!is_service_running('xgate')) {
	    xgate_start();
        } else {
	    xgate_reinit();
	}
}

function xgate_stop()
{
    log_error("Halting XGate");
    mwexec(XGATERC . " stop");
}

function xgate_start()
{
    log_error("Starting XGate");
    mwexec(XGATERC . " start");
}

function xgate_mon()
{
    log_error("Starting XGate Service Monitor");
    mwexec_bg(XGATEMON);
}

function xgate_restart()
{
    log_error("Restarting XGate");
    mwexec(XGATERC . " restart");
}

function xgate_reinit()
{

//	either reinit or kill works. pick your preference
//    mwexec(XGATE . " reinit");
    if ( file_exists(XGATE_PID) ) {
        log_error("Resyncing XGate");
	$cmd = "cat " . XGATE_PID;
	system("kill -USR1 `$cmd`");
    } else {
	xgate_start();
    }
}

function xgate($what)
{
    mwexec(XGATE . " " . $what);
}

function xgate_rotate_logs()
{
    xgate("rotate");
}

function xgate_start_transfer()
{
    xgate("go");
}

function xgate_stop_transfer()
{
    xgate("stop");
}

function xgate_break_transfer()
{
    xgate("break");
}

function xgate_purge_restart()
{
    xgate("purge");
}

function get_primary_users()
{
	global $config;
	$xgate = $config['installedpackages']['xgate']['config'][0];
	$acct = $config['installedpackages']['xgateprimaryusers']['config'];

	$ret = array($xgate['gateway_username']);
        if ( count($acct) ) { 
	    foreach ($acct as $user) {
	       $ret[] = $user['username'];
	    }
        }
	
	return $ret;
}

function xgate_build_options($pkg, $what, $options, $values)
{
	$i = 0;
	foreach ($pkg['fields']['field'] as $field) {
		if ($field['fieldname'] == $what)
			break;
		$i++;
	}
	$field = &$pkg['fields']['field'][$i];

	for ($i = 0; $i < count($options); $i++)
		$field['options']['option'][] = array('name' => $options[$i], 'value' => $values[$i]);
}


define('LOGSHOW_BUFSIZE', '16384');

// X-BigMail: XID-1338441576-87919;test;Chris Pettersson <chris@xaxero.com>;192119;Thu, 31 May 2012 17:19:09 +1200;Royalties
// X-BigMail: XID-1338432441-50840;test;Chris Pettersson <chris@xaxero.com>;88528;Thu, 31 May 2012 14:47:02 +1200;Re: Xaxero
// X-BigMail: XID-1338431480-46168;test;Stelian Codarcea <scodarcea@isp-usa.com>;80469;Thu, 31 May 2012 02:36:58 +0000;Xaxero // INV # S1264 // QUEST OR CLIPPER ADVENTURER ???

function bm_retrieve($rows, $xids)
{
    $selxids = bm_drop($rows, $xids);
    foreach ( $selxids as $xid ) {
        bm_write_to_outbox($xid[1], $xid[0], "retrieve");
    }
}

function bm_delete($rows, $xids)
{
    $selxids = bm_drop($rows, $xids);
    foreach ( $selxids as $xid ) {
        bm_write_to_outbox($xid[1], $xid[0], "delete");
    }
}

function bm_drop($rows, $xids)
{
    $selxids = process_xids($rows, $xids);
    bm_drop_from_list($xids, $selxids);
    return $selxids;
}

function bm_drop_from_list($xids, $selxids)
{

//   array_diff only works for single dimentionsal arrays
//   $diff = array_diff($xids, $selxids);

   foreach ($selxids as $selxid) {
	$keys = array_keys($xids);
   	foreach ($keys as $key ) {
            if ( $selxid[0] == $xids[$key][0] ) {
		unset($xids[$key]);
	    }
	}
   }

   $out = "";
   foreach ($xids as $xid ) {
       $line = trim(implode(";", $xid));
       $out .= $line . "\n";
   }

   safefilerewrite(XGATE_BIGMAIL,$out);
}

function process_xids($rows, $xids)
{
    $rowkeys = array_keys($rows);

    $selxid = array();
    foreach ( $rowkeys as $key ) {
	$xid = $xids[$key];
	if ( count($xid) ) {
	    $selxid[] = $xid;
        }
    }
    return $selxid;
}

function bm_make_list($mode,$selfpath)
{
    $bigmail = file(XGATE_BIGMAIL, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);

    $res .= "<table class='tabcont' width='100%' border='0' cellpadding='0' cellspacing='0'>\n";
    $res .= "<tr>\n";
    $res .= "<td class='listtopic'></td>\n";
    $res .= "<td class='listtopic'>Size</td>\n";
    $res .= "<td class='listtopic'>To</td>\n";
    $res .= "<td class='listtopic'>From</td>\n";
    $res .= "<td class='listtopic'>Date</td>\n";
    $res .= "<td wrap class='listtopic'>Subject</td>";
    $res .= "</tr>\n";

    $cnt = 0;
    foreach ($bigmail as $line) {
	$line = str_replace("X-BigMail: ", "", $line);
        $row = explode(";", $line);
	$size = htmlspecialchars($row[3]);
	$to = htmlspecialchars($row[1]);
	$from = htmlspecialchars($row[2]);
        $date = htmlspecialchars($row[4]);
	$subject = htmlspecialchars($row[5]);
	$pos = strpos($subject,"=?");
        if ( $pos !== false && $pos == 0 ) {
	   $subject = "UTF ENCODED";
 	}

	$name = "bm_row_" . $cnt++;
	$checked = ( $mode == "select" ) ? "checked=\"yes\"" : "";
        $res .= "<tr><td class='listlr'><input type=\"checkbox\" name=\"$name\" value=\"on\" $checked/></td>\n";
        $res .= "<td class='listr'>$size</td>";
        $res .= "<td class='listr'>$to</td>";
        $res .= "<td class='listr'>$from</td>";
        $res .= "<td class='listr'>$date</td>";
        $res .= "<td class='listr'>$subject</td></tr>";

	$bm_rows[$name] = $row;
	
    }

    $res .= "</table>";
    return array($res, $bm_rows);
}

function xgate_logrep( $filename, $lncount, $reverse )
{
    $res = "";
    $cont = "";

//    $lncount = $lncount ? $lncount : 50;
    $reverse = $reverse ? $reverse : false;

    if (file_exists($filename)) {
        $fh = fopen($filename, "r");
        if ($fh) {
  	    do { 
          	$cont .= fread($fh,  LOGSHOW_BUFSIZE);
	    } while ( !feof($fh) );
            fclose($fh);
        }

        $cont = explode( "\n", $cont );
	if ( $lncount > 0 ) {
	    $cont = array_slice($cont, -$lncount);
	}

        if ($reverse) $cont = array_reverse( $cont );

        $res .= "<table class='tabcont' width='100%' border='0' cellpadding='0' cellspacing='0'>\n";
        if ( $lncount > 0 ) {
            $res .= "<tr><td nowrap class='listtopic' colspan='2'>Last {$lncount} entries</td></tr>\n";
	}
        foreach($cont as $cn) {
            $cn = trim($cn);
	    if ( strlen($cn) ) {
	        $res .= "<tr><td nowrap class='listlr'>{$cn}</td></tr>";
	    }
        }
        $res .= "</table>";
    }

    return $res;
}

if(!function_exists('str_split')) {
  function str_split($string, $split_length = 1) {
    $array = explode("\r\n", chunk_split($string, $split_length));
    array_pop($array);
    return $array;
  } 
}

function safefilerewrite($fileName, $dataToSave)
{    
    if ($fp = fopen($fileName, 'w')) {
        $startTime = microtime();
        do  {
           $canWrite = flock($fp, LOCK_EX);
           // If lock not obtained sleep for 0 - 100 milliseconds, to avoid collision and CPU load
           if(!$canWrite) usleep(round(rand(0, 100)*1000));
        } while ((!$canWrite)and((microtime()-$startTime) < 1000));

        //file was locked so now we can store information
        if ($canWrite) {
            fwrite($fp, $dataToSave);
            flock($fp, LOCK_UN);
        }
        fclose($fp);
    }
}

function write_php_ini($array, $file)
{
    $res = array();
    if ( !$array ) {
        return;
    }
    foreach($array as $key => $val) {
        if(is_array($val))  {
            $res[] = "[$key]";
            foreach($val as $skey => $sval) {
//		$res[] = "$skey = ".(is_numeric($sval) ? $sval : '"'.$sval.'"');
		$res[] = "$skey=".$sval;
	    }
        } else {
//	    $res[] = "$key = ".(is_numeric($val) ? $val : '"'.$val.'"');
	    $res[] = "$key=".$val;
        }
    }
    safefilerewrite($file, implode("\r\n", $res) . "\r\n");
}

function is_validuid($id)
{
    if ( empty($id) ) 
        return false;
    
    if ( strlen($id) < 3 ) 
        return false;

    if ( preg_match('/^[a-zA-Z0-9._-]+$/', $id) ) 
        return true;

    return false;
}

function is_validpw($pw)
{
    if ( empty($pw) )
        return false;

    if ( strlen($pw) < 6 ) 
        return false;
	
    return true;
}

function is_uint($val)
{
    $t = intval(trim($val));
    return ( is_int($val) && $val > 0 );
}

function xg_parse_ini_file($filename)
{
	if ( !file_exists($filename) )
	    return array();

	$f = file ($filename, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
	$conf = array();
	$p = &$conf;
	for ( $i = 0; $i < count($f); $i++ ) {
	    $line = trim($f[$i]);
	    if ( preg_match('/\[.*\]/', $line) ) {
	     	$line = str_replace("[","",$line);
	     	$line = str_replace("]","",$line);

		$conf[$line] = array();
		$p = &$conf[$line];
	    } else {
	        $val = explode("=", $line, 2);
		if ( count($val) == 2 ) {
		    $p[$val[0]] = "$val[1]";
		}
	    }
	}
	return $conf;
}

//
// big mail routines
//

function bm_write_to_outbox($from, $subject, $msg)
{
    global $config;
    $conf = xg_parse_ini_file(XGATE_INI);
    if ( $conf['Misc']['hiprioritymsg'] != 1 ) {
        $conf['Misc']['hiprioritymsg'] = 1;
        write_php_ini($conf, XGATE_INI);
    }

    $xgate = $config['installedpackages']['xgate']['config'][0];
    $domain = $xgate['sat_domain'];
    $to = "mailbot@{$domain}";
    $from = "$from@{$domain}";

    $mail = <<<EOD
To: $to
From: $from
Subject: $subject

$msg


EOD;

    $filename = XGATE_OUTBOX . uniqid(rand(), false) . '.Mail';
    $fp = fopen($filename, 'w');
    fwrite($fp, $mail);
    fclose($fp);
}

function bm_inbound_cutoff($from,$insz)
{
    $size = "size $insz";
    bm_write_to_outbox($from, $size, $size);
}

function bm_move_back($from, $insz)
{
    $move = "move $insz";
    bm_write_to_outbox($from, $move, $move);
}

function bm_sync($from)
{
    bm_write_to_outbox($from, 'sync', 'sync');
}

function bm_purge_inbox($from)
{
    $purge = 'purge inbox';
    bm_write_to_outbox($from, $purge, $purge);
}

function bm_purge_bm($from)
{
    $purge = 'purge bigmail';
    bm_write_to_outbox($from, $purge, $purge); 
}

function bm_purge_inbox_with_cutoff($from,$purgectoff)
{
    $purge = "purge $purgectoff";
    bm_write_to_outbox($from, $purge, $purge);
}

function bm_purge_restart_files()
{
    $files = array_merge(glob(XGATE_OUTBOX . "*"), glob(XGATE_TMP . "*"));
    foreach($files as $file) unlink($file);
}


//
// spool maintenance functions
//

if ( !function_exists( "scandir" ) ) {
    function scandir($dir, $sort = 0) {
        $dirmap = array();

        if(!is_dir($dir)) {
            trigger_error("lib::scandir($dir): failed to open dir: Invalid argument", E_USER_WARNING);
            return false;
        }

        $dir = opendir($dir);

        while (false !== ($file = readdir($dir)))
            $dirmap[] = $file;

        closedir($dir);

        ($sort == 1) ? rsort($dirmap) : sort($dirmap);

        return $dirmap;
    }
}

function xgate_scan_user_spool($mailbox)
{
    $mpath = XGATE_SPOOL . $mailbox;
    $ret = array();

    if ( file_exists($mpath) ) {
        $root = array_diff(scandir($mpath), array('.','..','tmp'));
        foreach ($root as $file) {
    	    $path = $mpath . "/" . $file;
	    if ( is_file($path) ) {
	        $ret[] = $path;
	    } else {
	        $ret = array_merge($ret, xgate_scan_user_spool($mailbox . "/" . $file));
	    }	    
        }
    }
    return $ret;
}

function mailbox_delete($rows, $xids)
{
    $selxids = process_xids($rows, $xids);
    foreach ( $selxids as $xid ) {
        if ( file_exists($xid) ) {
            unlink($xid);
        }
    }
}

function mailbox_make_list($mode,$mailbox,$selfpath)
{
    $res .= "<table class='tabcont' width='100%' border='0' cellpadding='0' cellspacing='0'>\n";
    $res .= "<tr>\n";
    $res .= "<td class='listtopic'></td>\n";
    $res .= "<td class='listtopic'>Size</td>\n";
    $res .= "<td class='listtopic'>To</td>\n";
    $res .= "<td class='listtopic'>From</td>\n";
    $res .= "<td class='listtopic'>Date</td>\n";
    $res .= "<td wrap class='listtopic'>Subject</td>";
    $res .= "</tr>\n";

    $spool = xgate_scan_user_spool($mailbox);

    $cnt = 0;
    foreach ($spool as $path) {

	$size = htmlspecialchars(filesize($path));

	exec("grep -i '^date: ' $path", $adate);
	exec("grep -i '^subject: ' $path", $asubject);
	if ( strcasecmp($mailbox,"outbox") == 0 ) {
	    exec("grep -i '^to: ' $path", $ato);
	    $to = htmlspecialchars(substr(trim($ato[0]), strlen("to: ")));
	    $from = $mailbox;
	    if ( empty($to) ) {
	        $to = "unspecidied-recipient";
	    }
	} else {
  	    exec("grep -i '^from: ' $path", $afrom);
   	    $base = substr($path, strlen(XGATE_SPOOL));
	    $users = explode('/', $base);
	    $to = htmlspecialchars($users[0]);
	    if ( count($users) > 2 ) {
	        $to .= "+" . $users[1];
	    }
	    $from = htmlspecialchars(substr(trim($afrom[0]), strlen("from: ")));
	}

        $date = htmlspecialchars(substr(trim($adate[0]), strlen("date: ")));
	$subject = htmlspecialchars(substr(trim($asubject[0]), strlen("subject: ")));

	unset($afrom);
	unset($adate);
	unset($asubject);

	$pos = strpos($subject,"=?");
        if ( $pos !== false && $pos == 0 ) {
	   $subject = "UTF ENCODED";
 	}

	$name = "bm_row_" . $cnt++;
	$checked = ( $mode == "select" ) ? "checked=\"yes\"" : "";
        $res .= "<tr><td class='listlr'><input type=\"checkbox\" name=\"$name\" value=\"on\" $checked/></td>\n";
        $res .= "<td class='listr'>$size</td>";
        $res .= "<td class='listr'>$to</td>";
        $res .= "<td class='listr'>$from</td>";
        $res .= "<td class='listr'>$date</td>";
        $res .= "<td class='listr'>$subject</td></tr>";

	$bm_rows[$name] = $path;
    }

    $res .= "</table>";
    return array($res, $bm_rows);
}

function xgate_scan_spool()
{
    $exclude = array('.','..','database','tmp','OutBox');
    $root = array_diff(scandir(XGATE_SPOOL), $exclude);
    
    $spool = array();
    foreach ($root as $puser) {
	$spool[] = XGATE_SPOOL . $puser;
        $proot = array_diff(scandir(XGATE_SPOOL . $puser), $exclude);
	foreach ( $proot as $suser ) {
	    $sroot = XGATE_SPOOL . $puser . "/" . $suser;
	    if ( is_dir($sroot) === true) {
	        $spool[] = $sroot;
	    }
	}
    }
    return $spool;
}

function xgate_scan_tdahdb()
{
    $root = array_diff(scandir(T_DAH_DB), array('.','..','cached_templates', 'index.php', 'system_admin'));
    $dir = array();
    foreach ( $root as $file ) {
        $dir[] = T_DAH_DB . $file;
    }
    return $dir;
}

function xgate_purge_spool()
{
	global $g, $config;
	$settings = $config['installedpackages']['xgatesubaccounts']['config'];
	
	$primaryusers = get_primary_users();

	$dirs = array();

        if ( count($primaryusers) ) {
	    foreach ( $primaryusers as $puser ) {
	        $dirs[] = XGATE_SPOOL . $puser;
	        $dirs[] = T_DAH_DB . $puser . "_localhost";

		if ( count($settings) ) {
	            foreach ( $settings as $acct ) {
	                if ( $acct['xgatesubaccountsprimary'] == $puser ) {
		            $dirs[] = XGATE_SPOOL . $puser . "/" . $acct['username'];
		            $dirs[] = T_DAH_DB . $puser . "_" . $acct['username'] . "_localhost";
		        }
	            }
		}
	    }
	}

	$spool = xgate_scan_spool();
	$tdah = xgate_scan_tdahdb();

        $td = array_diff($tdah, $dirs);
	$sd = array_diff($spool, $dirs);

	$del = array_merge($td, $sd);

/*
	print_r("<p/>spool=");
	print_r($spool);
	print_r("<p/>dirs=");
	print_r($dirs);
	print_r("<p/>tdah=");
	print_r($tdah);
	print_r("<p/>del=");
	print_r($del);
*/
	foreach ( $del as $file ) {
	    if ( file_exists($file) && strpos($file, XGATE_SPOOL) !== false ) {
	    	system("rm -rf $file");
	    }
	}
}

function xgate_install_cron($should_install, $when) 
{
	global $config, $g;
	if($g['booting']==true) 
		return;
	$is_installed = false;
	if(!$config['cron']['item'])
		return;
	$x=0;
	foreach($config['cron']['item'] as $item) {
		if(strstr($item['task_name'], "xgate_rotate_logs")) {
			$is_installed = true;
			break;
		}
		$x++;
	}
	if($is_installed == true && $x > 0) {
	    unset($config['cron']['item'][$x]);
	    parse_config(true);
	    write_config();
	}

	switch($should_install) {
		case true:
			$cron_item = array();
			$cron_item['task_name'] = "xgate_rotate_logs";
			$cron_item['minute'] = "0";
			$cron_item['hour'] = "0";
			if ( $when == "monthly" ) {
			    $cron_item['mday'] = "*/1";
			} else {
			    $cron_item['mday'] = "*";
			}
			$cron_item['month'] = "*";
			if ( $when == "weekly" ) {
			    $cron_item['wday'] = "*/1";
			} else {
			    $cron_item['wday'] = "*";
			}
			$cron_item['who'] = "root";
			$cron_item['command'] = XGATE . " rotate";		
			$config['cron']['item'][] = $cron_item;
			parse_config(true);
			write_config("XGate Log Rotation");
			configure_cron();
		break;
		case false:
   		    if($is_installed == true) {
			configure_cron();
		    }
		break;
	}
}

function xgate_spool_tgz()
{
    xgate_backup_config_section('xgateconnection');
    xgate_backup_config_section('xgatefilters');
    xgate_backup_config_section('xgateprimaryusers');
    xgate_backup_config_section('xgatesubaccounts');

    $tname = tempnam("/tmp", "TAR");
    
    mwexec("/usr/bin/tar -czf $tname -C " . XGATE_VAR . " .");

    return $tname;
}

function xgate_backup_config_section($section)
{
    global $config;
    $new_section = &$config['installedpackages'][$section];
    $xmlconfig = dump_xml_config($new_section, $section);
    $xmlconfig = str_replace("<?xml version=\"1.0\"?>", "", $xmlconfig);
    $path = XGATE_VAR . $section . '.xml';
    file_put_contents($path, $xmlconfig);
}

function xgate_restore_config_section($section)
{
    global $config, $g;

    $path = XGATE_VAR . $section . '.xml';

    if ( !file_exists($path) ) {
        return false;
    }

    if ( count(file($path)) > 3 ) {
        $section_xml = parse_xml_config($path, $section);
        $config['installedpackages'][$section] = $section_xml;
    }

    return true;
}

function xgate_spool_restore($file)
{
    if ( !file_exists($file) ) {
        return "Failed to restore configuration. $file does not exist";
    }

    exec("/usr/bin/tar -tzf $file -C " . XGATE_VAR . " ./XGate.Ini", $out);

    if ( strpos($out[0], "XGate.Ini") === false ) {
        return "Failed to restore configuration. $file is not a valid archive";
    }

    mwexec("/usr/bin/tar -xzf $file -C " . XGATE_VAR);

    if ( !xgate_restore_config_section('xgateconnection')   ||
    	 !xgate_restore_config_section('xgatefilters')      ||
    	 !xgate_restore_config_section('xgateprimaryusers') ||
    	 !xgate_restore_config_section('xgatesubaccounts') ) {
	return "Failes to restore configuraton. Some of the configuration files are missing";
    }

    write_config("Restored mail config");

    xgate_resync();
    xgate_purge_spool();
    return true;
}

function xgate_upgrade($file)
{
    if ( !file_exists($file) ) {
        return "Failed to restore configuration. $file does not exist";
    }

    exec("/usr/bin/tar -tzf $file -C " . XGATE_VAR . " ./usr/local/gmnbin/xgate", $out);

    if ( strpos($out[0], "xgate") === false ) {
        return "Failed to restore configuration. $file is not a valid archive";
    }

    xgate_stop();
    mwexec("/usr/bin/tar -xzf $file -C /");
    xgate_start();
    return true;
}

function xgate_install()
{
    global $config;

    if ( file_exists(XGATERC) ) {
        xgate_stop();
    }

    $packages = &$config['installedpackages']['package'];
    $isXGatePackage = false;
    foreach ( $packages as $package ) {
        if ( $package['name'] == 'xgate' ) {
	   $isXGatePackage = true;
	}
    }

    if ( !$isXGatePackage ) {
        $xgatepkg['name']                        = 'xgate';
        $xgatepkg['descr']                       = 'High satellite email service.';
        $xgatepkg['website']                     = 'http://www.globalmarinenet.com/';
        $xgatepkg['category']                    = 'Network';
        $xgatepkg['version']                     = '4.0';
        $xgatepkg['status']                      = 'Stable';
        $xgatepkg['required_version']            = '1.2.3';
        $xgatepkg['maintainer']                  = 'support@globalmarinenet.com';
	$xgatepkg['depends_on_package_base_url'] = 'http://files.pfsense.org/packages/7/All/';
	$xgatepkg['config_file']                 = 'http://www.globalmarinenet.com/packages/config/xgate/xgate.xml';
	$xgatepkg['configurationfile']           = 'xgate.xml';

	$packages[] = $xgatepkg;
    }

    $pkgs =  &$config['installedpackages'];
    if ( !isset($pkgs['xgate']) ) {
        $xgate = &$pkgs['xgate']['config'][0];
        $xgate['misc_enabled']               = 'off';
	$xgate['sat_domain']                 = 'redportglobal.com';
	$xgate['tracking_automatic_updates'] = '60';
	$xgate['webmail_banner']             = 'RedPort Webmail';
	$xgate['webmail_signature']          = 'Sent via Satellite Phone using RedPort Email';
	$xgate['mail_pop3port']              = '110';
	$xgate['mail_smtpport']              = '25';
	$xgate['mail_logrotate']             = 'weekly';
	$xgate['misc_logdepth']              = '4';
    }

    $menus = &$config['installedpackages']['menu'];
    $isXGateMenu = false;
    foreach ( $menus as $menu ) {
        if ( $menu['name'] == 'RedPort Email Server' ) {
	    $isXGateMenu = true;
	}
    }
    
    if ( !$isXGateMenu ) {
        $xgatemenu['name']        = 'RedPort Email Server';
        $xgatemenu['tooltiptext'] = 'Modify RedPort email settings';
        $xgatemenu['section']     = 'Services';
        $xgatemenu['url']         = '/pkg_edit.php?xml=xgate.xml&amp;id=0';
	
	$menus[] = $xgatemenu;
    }
    
    $services = &$config['installedpackages']['service'];
    $isXGateService = false;
    foreach ( $services as $service ) {
        if ( $service['name'] == 'xgate' ) {
	    $isXGateService = true;
	}
    }

    if ( !$isXGateService ) {
        $xgatesrv['name']        = 'xgate';
        $xgatesrv['rcfile']      = 'xgate.sh';
        $xgatesrv['executable']  = 'xgate';
        $xgatesrv['description'] = 'RedPort Email Server Service';
	
	$services[] = $xgatesrv;
    }

    $vhosts = &$config['installedpackages']['vhosts']['config'];    
    $isXGateVhost = false;
    
    foreach ( $vhosts as $vhost ) {
    	if ( $vhost['description'] == 'Redport Webmail Client' ) {
	    $isXGateVhost = true;
	}
    }
    
    if ( !$isXGateVhost ) {
        $xgatevh['host'] = "";
        $xgatevh['ipaddress'] = "";
        $xgatevh['port'] = "8888";
	$xgatevh['directory'] = 'webmail';
        $xgatevh['certificate'] = "";
        $xgatevh['privatekey'] = "";
        $xgatevh['enabled'] = "true";
        $xgatevh['description'] = "Redport Webmail Client";
	
	$vhosts[] = $xgatevh;
    }
	
    if ( !file_exists(XGATE_VAR) )    mkdir(XGATE_VAR);
    if ( !file_exists(XGATE_SPOOL) )  mkdir(XGATE_SPOOL);
    if ( !file_exists(XGATE_OUTBOX) ) mkdir(XGATE_OUTBOX);
    if ( !file_exists(XGATE_TMP) )    mkdir(XGATE_TMP);
    
    mwexec("/bin/cp -r " . T_DAH_DB_DIST . " " . XGATE_SPOOL );
    
    create_ini_files();

    write_config("Installed XGate");

    xgate_resync();
    xgate_purge_spool();

    vhosts_sync_package();
    mwexec("/usr/local/etc/rc.d/vhosts-http.sh restart");

    xgate_mon();
    xgate_start();
}

function create_ini_files()
{

    $xgatelist = <<<EOD
;
; XGate Mailing Lists file. 
;
[MailingList]


EOD;

    if ( !file_exists(XGATE_LIST_INI) ) {
    	file_put_contents(XGATE_LIST_INI,$xgatelist);
    }

    $xgatealias = <<<EOD
;
; XGate alias file. 
;
[Alias]
; alaias=realuser

EOD;

    if ( !file_exists(XGATE_ALIAS_INI) ) {
        file_put_contents(XGATE_ALIAS_INI,$xgatealias);
    }

    $netini = <<<EOD
;
; Updated Nov 15, 2010
[HOSTS]
ServerNC.gmn-usa.com=208.86.227.175
ServerTX.gmn-usa.com=204.109.60.97
xgate.gmn-usa.com=208.86.227.175
xgate.ocens.net=208.86.227.175
xgate.globalsatellitemail.com=208.86.227.175
xgate.fmb.tesacom.net=208.86.227.175
xgatedns.gmn-usa.com=xgatedns.gmn-usa.com

[PORTS]
23=23
80=80
443=443
3333=3333

[XWEB]
xweb.gmn-usa.com=208.86.225.42
xweb2.gmn-usa.com=204.109.59.210

[ISPS]
; connection types 0-23 can not be rearranged due to compatibility issues with versions of xgate
; less than 3.54. types can be removed but should not reordered.
0=Network Connection
1=Globalstar
2=Iridium Direct Connect
3=INMARSAT
4=Other Satellite
5=HF Pactor
6=ISDN
7=Dialup
8=Cell Phone Dialup
10=StratosNet (Inmarsat, Iridium, MSAT)
11=Verizon Packet Network (cell phone)
16=Async MSAT USA #1
17=Async MSAT USA #2
22=Async MSAT CAN #1
23=Async MSAT CAN #2
; connection times greater than 24 can be rearranged anyway we want.  However, the list needs to
; be cooridinated with the XGate server for accurate type reporting in the log files.
24=Iridium RUDICS
25=Iridium Crew (blue SIM)
26=Iridium Crew RUDICS (blue SIM)
27=Telstra Iridium Direct Internet
28=Vizada Mini-M
29=INMARSAT MPDS
30=Thuraya
31=Iridium OpenPort
32=HNS BGAN
33=Explorer BGAN(not 100/110)
34=Explorer BGAN(100/110)
35=Sabre1
36=JRC Fleet Broadband
37=Sailor Fleet Broadband
38=Iridium Wifi
39=webXacc Iridium Direct Internet
40=webXacc Iridium OpenPort
41=webXacc Isatphone
42=webXacc Globalstar
43=webXacc Thuraya
44=webXacc MSAT USA
45=webXacc MSAT CAN

[webXacc MSAT USA]
ipproto=wxa
dialno=2404820105
ipdialchat=AT+SPEED=4800 OK AT+AUTH=$identity@ocens.net OK AT+KEY=$password OK AT+PHONE=$dialno OK ATDT=GENERIC OK
ipdischat=ATH0 OK
ipdialno=192.168.10.1
ipdialport=5454
ipconto=90
ipxferto=90
ipbufsz=1024

[webXacc MSAT CAN]
ipproto=wxa
dialno=6137326513
ipdialchat=AT+SPEED=4800 OK AT+AUTH=$identity@ocens.net OK AT+KEY=$password OK AT+PHONE=$dialno OK ATDT=GENERIC OK
ipdischat=ATH0 OK
ipdialno=192.168.10.1
ipdialport=5454
ipconto=90
ipxferto=90
ipbufsz=1024

[Iridium Wifi]
ipproto=ccp
dialno=008816000025
ipdialno=192.168.0.1
ipdialport=8890
ipconto=60
ipxferto=60
ipbufsz=1024

[webXacc Iridium OpenPort]
ipproto=wxa
ipdialchat=ATDT=OPENPORT OK
ipdischat=ATH0 OK
ipdialno=192.168.10.1
ipdialport=5454
ipconto=60
ipxferto=300
ipbufsz=0

[webXacc Isatphone]
ipproto=wxa
dialno=28
ipdialchat=ATDT=ISATPHONE OK
ipdischat=ATH0 OK
ipdialno=192.168.10.1
ipdialport=5454
ipconto=90
ipxferto=90
ipbufsz=1024

[webXacc Iridium Direct Internet]
ipproto=wxa
dialno=008816000025
ipdialchat=ATDT=IRIDIUM OK
ipdischat=ATH0 OK
ipdialno=192.168.10.1
ipdialport=5454
ipconto=60
ipxferto=60
ipbufsz=1024

[webXacc Globalstar]
ipproto=wxa
dialno=#777
ipdialchat=ATDT=GLOBALSTAR OK
ipdischat=ATH0 OK
ipdialno=192.168.10.1
ipdialport=5454
ipconto=60
ipxferto=60
ipbufsz=1024

[webXacc Thuraya] 
ipproto=wxa
dialno=*1722#
ipdialchat=ATDT=THURAYA OK
ipdischat=ATH0 OK
ipdialno=192.168.10.1
ipdialport=5454
ipconto=60
ipxferto=60
ipbufsz=1024

[HNS BGAN]
ipproto=at
ipdialchat=AT+CREG=1 OK AT+CREG? 1,1 AT+CGDCONT=11,"IP","bgan.inmarsat.com","",0,1,,,"$localip" OK  AT+CGEQREQ=11,3,512,512,512,512,2,0,"0E0","0E0",3,500,0 OK AT+CGACT=1,11 OK
ipdischat=AT+CGACT=0,11 OK
ipdialno=192.168.128.100
ipdialport=1829
ipconto=60
ipxferto=60
ipbufsz=0

[Explorer BGAN(not 100/110)]
ipproto=at
ipdialchat=AT+CREG=1 OK AT+CGEQREQ=1,3 OK AT+CGACT=1,1 OK
ipdischat=AT+CGACT=0,1 OK
ipdialno=192.168.0.1
ipdialport=5454
ipconto=60
ipxferto=60 
ipbufsz=0

[Explorer BGAN(100/110)]
ipproto=at
ipdialchat=AT+CREG=1 OK AT+CGEQREQ=1,3 OK AT+CGACT=1,1 OK
ipdischat=AT+CGACT=0,1 OK
ipdialno=192.168.0.1
ipdialport=9998
ipconto=60
ipxferto=60 
ipbufsz=0

[Sabre1]
ipproto=at
ipdialchat=AT+CGREG=1 OK AT_IASETMODE=1,1 OK AT_IANAT=1 OK AT+CGDCONT=10,"IP","BGAN.INMARSAT.COM",,0,1 OK AT_IAPPPCONNECT=1,10 OK
ipdischat=AT_IAPPPCONNECT=0,10 OK
ipdialno=192.168.1.35
ipdialport=9998
ipconto=90
ipxferto=60 
ipbufsz=0

[JRC Fleet Broadband]
ipproto=at
ipdialchat=AT+CREG=1 OK AT+CGEQREQ=1,3 OK AT+CGACT=1,1 OK
ipdischat=AT+CGACT=0,1 OK
ipdialno=192.168.128.100
ipdialport=1829
ipconto=60
ipxferto=60 
ipbufsz=0

[Sailor Fleet Broadband]
ipproto=at
ipdialchat=AT+CREG=1 OK AT+CGEQREQ=1,3 OK AT+CGACT=1,1 OK
ipdischat=AT+CGACT=0,1 OK
ipdialno=192.168.0.1
ipdialport=5454
ipconto=60
ipxferto=60 
ipbufsz=0

[Iridium OpenPort]
ipproto=network
ipconto=60
ipxferto=300
ipbufsz=0
override=no
network=yes

[Network Connection]
ipproto=network
ipconto=15
ipxferto=300
ipbufsz=0
override=no
network=yes

[Globalstar]
dialno=#777

[Thuraya]
dialno=1722

[INMARSAT MPDS]
dialno=**94#

[Vizada Mini-M]
override=no
username=inmarsat
password=inmarsat
dialno=67#

[Telstra Iridium Direct Internet]
override=no
username=telstra
password=telstra
dialno=66666

[Iridium RUDICS]
username=gmn
password=gmn
dialno=0088160000606

[Iridium Crew RUDICS (blue SIM)]
username=gmn
password=gmn
dialno=69888160000606

[Iridium Crew (blue SIM)]
username=iridium
password=iridium
dialno=6988816000025

[Iridium Direct Connect]
username=iridium
password=iridium

[INMARSAT]
username=inmarsat
password=inmarsat
ipconto=60
ipxferto=120

[StratosNet (Inmarsat, Iridium, MSAT)]
username=guest
password=guest

[Verizon Packet Network (cell phone)]
override=no
username=qnc
password=qnc
dialno=#777

[Async MSAT USA #1]
override=no
username=$identity@ocens.net
password=$password
dialno=2404820105

[Async MSAT USA #2]
override=no
username=$identity@ocens.net
password=$password
dialno=2404271028

[Async MSAT CAN #1]
override=no
username=$identity@ocens.net
password=$password
dialno=6137326513

[Async MSAT CAN #2]
override=no
username=$identity@ocens.net
password=$password
dialno=6135477639

; 
; old style ini file no longer supported after xgate 3.54
[ISP]
; HF Pactor XNET
Name9=xnetmail.com
Num9=XNET
ID9=xnet

; Globalstar
Name12=ocens.net
Num12=0013136377827
ID12=ocensmail
Name13=ocens.net
Num13=0016193310155
ID13=ocensmail

; Iridium
Name14=ocens.net
Num14=0016029260155
ID14=ocensmail
Name15=ocens.net
Num15=0016193310155
ID15=ocensmail

; MSat USA
Name16=ocens.net
Num16=7036471416
ID16=ocensmail
Name17=ocens.net
Num17=4435730277
ID17=ocensmail

; Inmarsat
Name18=ocens.net
Num18=0012153667827
ID18=ocensmail
Name19=ocens.net
Num19=0013136377827
ID19=ocensmail

; Genereral dialup
Name20=ocens.net
Num20=13136377827
ID20=ocensmail
Name21=ocens.net
Num21=16785383300
ID21=ocensmail

; MSat CAN
Name22=ocens.net
Num22=6137326513
ID22=ocensmail
Name23=ocens.net
Num23=6135477639
ID23=ocensmail

EOD;

     if ( !file_exists(XGATE_NET_INI) ) {
        file_put_contents(XGATE_NET_INI, $netini);
    }

    $xgateini = <<<EOD
[All]
Domain0=redportglobal.com
InboxName=OutBox
Users=XGate.User
AppLog=xgate.log
TmpDir=tmp
Alias=XGate.Alias
MailingList=XGate.Lists
[SMTP]
Port=25
Relay=1
SMTPLog=smtp.log
RelayNetAddr=0.0.0.0
[POP3]
Passwords=XGate.Pass
Port=110
PopLog=pop3.log
[Agent]
Agent=GMN
Tier1Dealer=GMN
AppName=XGate
CompanyName=Global Marine Networks, LLC
Domain=redportglobal.com
RegEmail=info@gmn-usa.com
RegFax=+865.681.5016
RegTel=+865.379.8723
RegUrl=http://www.globalmarinenet.com
SupEmail=support@gmn-usa.com
SupFax=+865.681.5016
SupTel=+865.379.8723
SupUrl=http://www.globalmarinenet.com
[Gateway]
username=undefined
password=undefined
xport=443
gateway=xgate.gmn-usa.com
failover=1
gateway2=ServerTX.gmn-usa.com
bTimeOutOverride=0
[Sat]
domain=redportglobal.com
relayflag=0
MailRelayAddr=0.0.0.0
[Mail]
pop3port=110
smtpport=25
poplog=1
smtplog=1
concurrency=0
blockrestart=1
autorestart=1
readpipe=1
writepipe=1
[Connection]
connection=Network Connection
type=0
ipdialoverride=0
leaveopen=0
useifopen=0
persist=0
[Misc]
allowplusbigmailonsend=0
usewhitelistonsend=0
bigmailnotifyenum=0
hiprioritymsg=0
enabled=0
logdepth=4
[MailFilter]
cutoffsize=50
outcutoffsize=300
[Tracking]
AutomaticUpdates=1
UpdateInterval=60

EOD;

       if ( !file_exists(XGATE_INI) ) {
         file_put_contents(XGATE_INI, $xgateini);
     }
}

function xgate_uninstall()
{
    global $config;

    xgate_stop();

    $pkgs =  &$config['installedpackages'];
    if ( isset($pkgs['xgate']) ) {
        unset($pkgs['xgate']);
    }

    if ( isset($pkgs['xgateconnection']) ) {
        unset($pkgs['xgateconnection']);
    }

    if ( isset($pkgs['xgatefilters']) ) {
        unset($pkgs['xgatefilters']);
    }
    
    if ( isset($pkgs['xgateprimaryusers']) ) {
        unset($pkgs['xgateprimaryusers']);
    }

    if ( isset($pkgs['xgatesubaccounts']) ) {
        unset($pkgs['xgatesubaccounts']);
    }

    $packages = &$config['installedpackages']['package'];
    $npackages = array();
    foreach ( $packages as $package ) {
        if ( $package['name'] != 'xgate' ) {
	    $npackages[] = $package;
	}
    }
    $packages = $npackages;

    $menus = &$config['installedpackages']['menu'];
    $nmenus = array();
    foreach ( $menus as $menu ) {
        if ( $menu['name'] != 'RedPort Email Server' ) {
	    $nmenus[] = $menu;
	}
    }
    $menus = $nmenus;
    
    $services = &$config['installedpackages']['service'];
    $nservices = array();
    foreach ( $services as $service ) {
        if ( $service['name'] != 'xgate' ) {
	    $nservices[] = $service;
	}
    }
    $services = $nservices;

    $vhosts = &$config['installedpackages']['vhosts']['config'];    
    $nvhosts = array();    
    foreach ( $vhosts as $vhost ) {
    	if ( $vhost['description'] != 'Redport Webmail Client' ) {
	    $nvhosts[] = $vhost;
	}
    }
    $vhosts = $nvhosts;

    $crontabs = &$config['cron']['item'];
    $ncrontabs = array();
    foreach($crontabs as $crontab) {
	if($crontab['task_name'] != "xgate_rotate_logs") {
	    $ncrontabs[] = $crontab;
	}
    }
    $crontabs = $ncrontabs;

    write_config("Uninstalling XGate");
    
    mwexec("/usr/local/etc/rc.d/vhosts-http.sh restart");

    mwexec("rm -rf " . XGATE_VAR);
    mwexec("rm -rf " . T_DAH);
    mwexec("rm -rf /usr/local/www/xgate");
    mwexec("rm -f /usr/local/pkg/xgate*");
    mwexec("rm -f /usr/local/gmnbin/xgate*");
    unlink("/usr/local/etc/rc.d/xgate.sh");
    system("kill -TERM `(ps uax | grep email_monitor.sh | grep -v grep | awk '{ print $2 }')` 1>/dev/null 2>&1");
    unlink("/usr/local/etc/rc.d/email_monitor.sh");
}
